import axios from 'axios'
import { Notification, MessageBox, Message, Loading } from 'element-ui'
import router from '@/router'
import store from '@/store'
import { getToken } from '@/utils/auth'
import errorCode from '@/utils/errorCode'
import { tansParams,judgmentNetworkReturnAddress } from "@/utils/ruoyi";

//判定http或者https
let urlText = window.location.href.split('://')[0]+'://';

//分锅器
let potType = false;

//弹窗状态开关
let messageData = null;

// 弹层数据
let loadingInstance = {};
let options = {
  spinner:"",
  background: 'rgba(255, 255, 255, 0.1)'
};
let loadingCount = 0;

axios.defaults.headers['Content-Type'] = 'application/json;charset=utf-8'
// 创建axios实例
const service = axios.create({
  // axios中请求配置有baseURL选项，表示请求URL公共部分
  baseURL: urlText+judgmentNetworkReturnAddress(),
  // 超时
  timeout: 10000
})
// request拦截器
service.interceptors.request.use(config => {
  if(config.type == 'video'){
    config.baseURL = localStorage.getItem('cameraExtranetAgent');
    potType = true;
  }else{
    potType = false;
  }
  // 弹层相关
  loadingInstance = Loading.service(options)
  // 是否需要设置 token
  const isToken = (config.headers || {}).isToken === false
  if (getToken() && !isToken) {
    config.headers['Authorization'] = 'Bearer ' + getToken() // 让每个请求携带自定义token 请根据实际情况自行修改
  }
  // get请求映射params参数
  if (config.method === 'get' && config.params) {
    let url = config.url + '?' + tansParams(config.params);
    url = url.slice(0, -1);
    config.params = {};
    config.url = url;
  }
  // 弹层相关
  loadingCount ++;
  return config
}, error => {
  // 弹层相关
    loadingCount --;
    if(loadingCount===0){
      loadingInstance.close();
    }
    console.log(error)
    Promise.reject(error)
})

// 响应拦截器
service.interceptors.response.use(res => {
    // 弹层相关
    loadingCount --;
    if(loadingCount===0) {
      loadingInstance.close();
    }
    // 未设置状态码则默认成功状态
    const code = res.data.code || 200;
    // 获取错误信息
    const msg = errorCode[code] || res.data.msg || errorCode['default']
    if (code == 401) {
      if(!localStorage.getItem('serviceType')){
        localStorage.setItem('serviceType','1')
        MessageBox.confirm('登录状态已过期，请重新登录', '系统提示', {
            confirmButtonText: '确定',
            showCancelButton:false,
            closeOnClickModal:false,
            cancelButtonText: '取消',
            type: 'warning'
          }
        ).then(() => {
          store.dispatch('LogOut').then(() => {
            localStorage.removeItem('serviceType');
            router.replace({
              path: '/login'
            })
            // location.href = '/index';
          })
          return
        }).catch(() => {});
      }
      // return Promise.reject('令牌验证失败')
    } else if (code == 403) {
      // if(!localStorage.getItem('serviceType')){
      //   MessageBox.confirm('没有权限，请联系管理员授权！', '系统提示', {
      //       confirmButtonText: '确定',
      //       showCancelButton:false,
      //       closeOnClickModal:false,
      //       cancelButtonText: '取消',
      //       type: 'warning'
      //     }
      //   ).then(() => {
      //   }).catch(() => {});
      // }
      // return Promise.reject('error')
      router.replace({
        path: '/403'
      })
    } else if (code === 500) {
      if(!messageData){
        messageData = Message({
          message: msg,
          type: 'error',
          offset:100
        })
        setTimeout(function(){
          messageData = null
        },1000);
      }
      return Promise.reject(new Error(msg))
    } else if (code == 303 || code == 530 || code == 503 || code == 504|| code == 205) {
      // 303-考试从定向   530-账号不存在   503-验证码错误  504特殊报错code 205智能门禁状态 用于报错后需要返回code进行后续操作
      return res.data
    } else if (code != 200) {
      Notification.error({
        title: msg
      })
      return Promise.reject('error')
    } else {
      // console.log("res",res.data)
      return res.data
    }
  },
  error => {
    // 弹层相关
    loadingCount --;
    if(loadingCount===0){
      loadingInstance.close();
    }
    console.log('err' + error)
    let { message } = error;
    if (message == "Network Error") {
      if(potType){
        message = "视频监控接口连接异常";
      }else{
        message = "服务连接异常";
      }
    }
    else if (message.includes("timeout")) {
      if(potType){
        message = "视频监控接口请求超时";
      }else{
        message = "系统接口请求超时";
      }
    }
    else if (message.includes("Request failed with status code")) {
      message = "系统接口" + message.substr(message.length - 3) + "异常";
    }
    if(!messageData){
      messageData = Message({
        message: message,
        type: 'error',
        duration: 5 * 1000,
        offset:100
      })
      setTimeout(function(){
        messageData = null
      },1000);
    }
    return Promise.reject(error)
  }
)

// 通用下载方法
export function download(url, params, filename) {
  return service.post(url, params, {
    transformRequest: [(params) => {
      return tansParams(params)
    }],
    headers: {
      'Content-Type': 'application/x-www-form-urlencoded'
    },
    responseType: 'blob'
  }).then((data) => {
    const content = data
    const blob = new Blob([content])
    if ('download' in document.createElement('a')) {
      const elink = document.createElement('a');
      elink.download = filename
      elink.style.display = 'none'
      elink.href = URL.createObjectURL(blob)
      document.body.appendChild(elink)
      elink.click()
      URL.revokeObjectURL(elink.href)
      document.body.removeChild(elink)
    } else {
      navigator.msSaveBlob(blob, filename)
    }
  }).catch((r) => {
    console.error(r)
  })
}
// 通用地址文件下载方法
export function downloadUrl(fileUrl,fleName) {
  const newUrl = urlText + judgmentNetworkReturnAddress() + fileUrl;
  const x = new XMLHttpRequest();
  x.open("GET", newUrl, true);
  x.responseType = "blob";
  x.onload = () => {
    const url = URL.createObjectURL(x.response);
    const a = document.createElement("a");
    a.href = url;
    a.download = fleName;
    a.target = "_blank";
    a.click();
  };
  x.send();
}


export default service
