'use strict'
const path = require('path')

const ip = '192.168.1.43'
const os = require('os')
const fs = require("fs")
//判断环境
let ifaces = os.networkInterfaces()
let localityIp = '', result = []
for(let dev in ifaces) {
  ifaces[dev].forEach(function(details) {
    if(localityIp === '' && details.family === 'IPv4' && !details.internal) {
      localityIp = details.address
      return;
    }
  })
}
let getFile = localityIp == ip?false:true
/******** 获取git版本信息 ********/
const gitHEAD = getFile ? fs.readFileSync('.git/HEAD', 'utf-8').trim() : false
const ref = getFile ? gitHEAD.split(': ')[1] : false
const develop = getFile ? gitHEAD.split('/')[2] : false
const gitVersion = getFile ? fs.readFileSync('.git/' + ref, 'utf-8').trim() : false
const buildEdition = develop && gitVersion ? develop + ': ' + gitVersion : '未配置'
const myDate = new Date()
const buildTime = myDate.getFullYear() +'-'+ (myDate.getMonth()+1) +'-'+ myDate.getDate() +' '+ myDate.getHours() +':'+ myDate.getMinutes();
const logsList = getFile ? fs.readFileSync('.git/logs/HEAD', 'utf-8').replace(/\r\n|\r/g, "\n").split("\n") : false
const buildUser = getFile ? logsList[logsList.length-2].split(" ")[2] : '未配置'

function resolve(dir) {
  return path.join(__dirname, dir)
}

const name = process.env.VUE_APP_TITLE || '实验室安全管理系统' // 网页标题

const port = process.env.port || process.env.npm_config_port || 80 // 端口

// vue.mqttConfig.js明
//官方vue.mqttConfig.js档 https://cli.vuejs.org/zh/config/#css-loaderoptions
// 这里只列一部分，具体配置参考文档
module.exports = {
  // 部署生产环境和开发环境下的URL。
  // 默认情况下，Vue CLI 会假设你的应用是被部署在一个域名的根路径上
  // 例如 https://www.ruoyi.vip/。如果应用被部署在一个子路径上，你就需要用这个选项指定这个子路径。例如，如果你的应用被部署在 https://www.ruoyi.vip/admin/，则设置 baseUrl 为 /admin/。
  publicPath: process.env.NODE_ENV === "production" ? "./" : "./",
  // 在npm run build 或 yarn build 时 ，生成文件的目录名称（要和baseUrl的生产环境路径一致）（默认dist）
  outputDir: 'dist',
  // 用于放置生成的静态资源 (js、css、img、fonts) 的；（项目打包之后，静态资源会放在这个文件夹下）
  assetsDir: 'static',
  // 是否开启eslint保存检测，有效值：ture | false | 'error'
  lintOnSave: process.env.NODE_ENV === 'development',
  // 如果你不需要生产环境的 source map，可以将其设置为 false 以加速生产环境构建。
  productionSourceMap: false,
  // webpack-dev-server 相关配置
  devServer: {
    host: '0.0.0.0',
    port: port,
    open: true,
    proxy: {
      // detail: https://cli.vuejs.org/config/#devserver-proxy
      [process.env.VUE_APP_BASE_API]: {
        target: `http://localhost:8080`,
        changeOrigin: true,
        pathRewrite: {
          ['^' + process.env.VUE_APP_BASE_API]: ''
        }
      }
    },
    disableHostCheck: true
  },
  //修改了sass版本从1.32.0 升级至1.39.0  以解决页面小图标偶发性乱码问题 以下css代码为同步增加
  css: {
    loaderOptions: {
      sass: {
        sassOptions: {
          outputStyle: 'expanded'
        }
      }
    }
  },
  configureWebpack: {
    name: name,
    resolve: {
      alias: {
        '@': resolve('src')
      }
    }
  },
  chainWebpack(config) {
    config.plugins.delete('preload') // TODO: need test
    config.plugins.delete('prefetch') // TODO: need test


    // 图片转base64
    // const imagesRule = config.module.rule('images');
    // imagesRule.uses.clear()        //清除原本的images loader配置
    // imagesRule
    //   .test(/\.(jpg|gif|png|svg)$/)
    //   .exclude
    //   .add(path.join(__dirname,"../node_modules"))//去除node_modules里的图片转base64配置
    //   .end()
    //   .use('url-loader')
    //   .loader('url-loader')
    //   .options({name:"img/[name].[hash:8].[ext]",limit: 1})


    // set svg-sprite-loader
    config.module
      .rule('svg')
      .exclude.add(resolve('src/assets/icons'))
      .end()
    config.module
      .rule('icons')
      .test(/\.svg$/)
      .include.add(resolve('src/assets/icons'))
      .end()
      .use('svg-sprite-loader')
      .loader('svg-sprite-loader')
      .options({
        symbolId: 'icon-[name]'
      })
      .end()

    config
      .when(process.env.NODE_ENV !== 'development',
        config => {
          //标记打包时间与当前git版本信息
          config.plugin('html').tap(args => {
            args[0].buildUser = buildUser;
            args[0].buildTime = buildTime;
            args[0].buildEdition = buildEdition;
            args[0].minify.removeComments = false;
            return args
          })
          config
            .plugin('ScriptExtHtmlWebpackPlugin')
            .after('html')
            .use('script-ext-html-webpack-plugin', [{
              // `runtime` must same as runtimeChunk name. default is `runtime`
              inline: /runtime\..*\.js$/
            }])
            .end()
          config
            .optimization.splitChunks({
            chunks: 'all',
            cacheGroups: {
              libs: {
                name: 'chunk-libs',
                test: /[\\/]node_modules[\\/]/,
                priority: 10,
                chunks: 'initial' // only package third parties that are initially dependent
              },
              elementUI: {
                name: 'chunk-elementUI', // split elementUI into a single package
                priority: 20, // the weight needs to be larger than libs and app or it will be packaged into libs or app
                test: /[\\/]node_modules[\\/]_?element-ui(.*)/ // in order to adapt to cnpm
              },
              echarts: {
                name: 'chunk-echarts',
                priority: 20,
                test: /[\\/]node_modules[\\/]_?echarts(.*)/
              },
              flv: {
                name: 'chunk-flv',
                priority: 20,
                test: /[\\/]node_modules[\\/]_?flv(.*)/
              },
              jsBeautify: {
                name: 'chunk-vue-js-beautify',
                priority: 20,
                test: /[\\/]node_modules[\\/]_?js-beautify(.*)/
              },
              jsencrypt: {
                name: 'chunk-vue-jsencrypt',
                priority: 20,
                test: /[\\/]node_modules[\\/]_?jsencrypt(.*)/
              },
              api: {
                name: 'chunk-api',
                test: resolve('src/api'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
              assets: {
                name: 'chunk-assets',
                test: resolve('src/assets'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
              commons: {
                name: 'chunk-commons',
                test: resolve('src/components'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
              directive: {
                name: 'chunk-directive',
                test: resolve('src/directive'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
              utils: {
                name: 'chunk-utils',
                test: resolve('src/utils'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
              comprehensive: {
                name: 'chunk-comprehensive',
                test: resolve('src/views/comprehensive'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
              creditViolation: {
                name: 'chunk-creditViolation',
                test: resolve('src/views/creditViolation'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
              dashboard: {
                name: 'chunk-dashboard',
                test: resolve('src/views/dashboard'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
              emergencyManagement: {
                name: 'chunk-emergencyManagement',
                test: resolve('src/views/emergencyManagement'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
              gasManage3_0: {
                name: 'chunk-gasManage3_0',
                test: resolve('src/views/gasManage3_0'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
              gasManage_syd: {
                name: 'chunk-gasManage_syd',
                test: resolve('src/views/gasManage_syd'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
              hazardManagement: {
                name: 'chunk-hazardManagement',
                test: resolve('src/views/hazardManagement'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
              hierarchicalControl: {
                name: 'chunk-hierarchicalControl',
                test: resolve('src/views/hierarchicalControl'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
              medicUniversity_3_1: {
                name: 'chunk-medicUniversity-3_1',
                test: resolve('src/views/medicUniversity-3_1'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
              mine: {
                name: 'chunk-mine',
                test: resolve('src/views/mine'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
              oneKeyOutfire: {
                name: 'chunk-oneKeyOutfire',
                test: resolve('src/views/oneKeyOutfire'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
              safetyEducationExam: {
                name: 'chunk-safetyEducationExam',
                test: resolve('src/views/safetyEducationExam'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
              secureAccess: {
                name: 'chunk-secureAccess',
                test: resolve('src/views/secureAccess'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
              securityCheck: {
                name: 'chunk-securityCheck',
                test: resolve('src/views/securityCheck'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
              studentViews: {
                name: 'chunk-studentViews',
                test: resolve('src/views/studentViews'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
              system: {
                name: 'chunk-system',
                test: resolve('src/views/system'), // can customize your rules
                priority: 0,
                minSize: 0,
                minChunks: 1,
                enforce:true,
                reuseExistingChunk: true
              },
            }
          })
          config.optimization.runtimeChunk('single'),
            {
              from: path.resolve(__dirname, './public/robots.txt'), //防爬虫文件
              to: './', //到根目录下
            }
        }
      )
  }
}
